/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */

#include "config.h"
#include <stdio.h>
#include <errno.h>
#include <assert.h>

#include <Xm/DialogS.h>
#include <Xm/PushBG.h>
#include <Xm/PushB.h>
#include <Xm/LabelG.h>
#include <Xm/PanedW.h>
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include "xmap.h"
#include "debug.h"

void open_db(void);
struct db_entry *read_db(int);
void close_db(void);

struct db_entry *dbelist = NULL;
struct db_entry *dbelast = NULL;

static FILE *fp;

struct db_entry *get_db_entry()
{
    struct db_entry *dbe;
    
    if (!(dbe = (struct db_entry *)malloc(sizeof(struct db_entry)))) {
	error_wait("Can't make map database entry!\n",errno); // exits
    }
    dbe->description = NULL;
    dbe->name = NULL;
    dbe->flags = 0;
    return dbe;
}

void kill_db_entry(struct db_entry *dbe)
{
    free(dbe);
}

void open_db()
{
    char *p;
    int n;
    struct db_entry *dbe;
    char bufp[256];

    if (!fp) {
	snprintf(bufp, 255, "%s/map.db", get_mapdir());
	if ( (fp = fopen(bufp, "ro")) == NULL )
	    error_wait("Can't open map database!\n",errno); // exits xmap
    }
    while (fgets(bufp, 255, fp)) {
	if (!strlen(bufp) || bufp[0] == '\n' || bufp[0] == ' '
	    || bufp[0] == '\t' || bufp[0] == '#')
	    continue;

	if (bufp[strlen(bufp) - 1] == '\n')
	    bufp[strlen(bufp) - 1] = '\0';

	p = bufp;
	while (*p && *p != ' ' && *p != '\t')
	    p++;

	*(p++) = '\0';
	while (*p && *p == ' ' && *p == '\t')
	    p++;

	// Now check for the .bsb extension
	if (strlen(bufp)>=4 && strcmp(&(bufp[strlen(bufp)-4]),".bsb")==0) {
	    char lb[256];
	    n = bsb_num_maps(bufp);
	    if (n) {
	      int i;
	      for (i = 0; i < n; i++) {
		  if (bsb_list_map(bufp, &dbe, i+1, n) != FALSE) 
		    LIST_INSERT_TAIL(&dbelist, &dbelast, dbe);
	      }
	    }
	    else continue;
	}
	else if (strlen(bufp)>=4 && strcmp(&(bufp[strlen(bufp)-4]),".shp")==0)
	  {
	    dbe = get_db_entry();
	    dbe->name = strdup(bufp);
	    dbe->description = strdup(p);
	    dbe->flags = M_SHAPE;	
	    LIST_INSERT_TAIL(&dbelist, &dbelast, dbe);
	  }
	else {
	    dbe = get_db_entry();
	    dbe->name = strdup(bufp);
	    dbe->description = strdup(p);
	    LIST_INSERT_TAIL(&dbelist, &dbelast, dbe);
	}
    }
}

struct db_entry *read_db(int mapno)
{
    static struct db_entry *dbe;
    static int curmapno;

    if (!dbe || (mapno < curmapno)) {
	dbe = dbelist;
	curmapno = 0;
    }

    while (dbe && (curmapno < mapno)) {
	curmapno++;
	dbe = dbe->next;
    }

    return dbe;
}

void close_db(void)
{
    struct db_entry *dbe = dbelist;
    struct db_entry *dbep;
    if (fp) {
	fclose(fp);
	fp = NULL;

	while (dbe) {
	    if (dbe->name) free(dbe->name);
	    if (dbe->description) free(dbe->description);
	    dbep = dbe;
	    dbe = dbe->next;
	    LIST_DELETE(dbep);
	    kill_db_entry(dbep);
	}
    }
}
