/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */
#include <stdio.h>
#include <assert.h>

#include <Xm/DialogS.h>
#include <Xm/ScrolledW.h>
#include <Xm/DrawingA.h>
#include <Xm/PushB.h>
#include <Xm/ArrowB.h>
#include <Xm/TextF.h>
#include <Xm/Text.h>
#include <Xm/Scale.h>
#include <Xm/Label.h>
#include <Xm/Frame.h>
#include <Xm/BulletinB.h>
#include <Xm/SelectioB.h>
#include <Xm/DrawnB.h>
#include <Xm/DragDrop.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#include "xmap.h"
#include "gps.h"
#include "mode.h"
#include "debug.h"

static Widget dialog = NULL;
static Widget bul;
static Widget ab1, ab2;
static int answer;
static int runing = 0;

extern XtAppContext app;
extern Widget toplevel;

#define HB 3
#define WB 4

#define BH 36
#define BW 36

#define H (HB*BH+18)
#define W (WB*BW)

static Widget bt[HB * WB];

static int offset = 0;
static int selected = -1;

#ifdef DND
static char dragTranslations[] =
"#override <Btn2Down>: StampStartDrag()";

XtTranslations stamp_drag_translations;
#endif

int place_object(int x, int y)
{
	double lat, lon;

	if (selected == -1) return 0;

	xytolatlon(x,y,&lat,&lon);

	create_pixmap_object(lat, lon, selected);
	return 1;
}

static redraw_stamp(int i)
{
    if (i + offset < NUMOBJ) {
	XtVaSetValues(bt[i],
		      XmNlabelPixmap, objects[i + offset].pmap,
		      NULL);
	if (i + offset == selected) {
	    XtVaSetValues(bt[i],
			  XmNpushButtonEnabled, False,
			  XmNshadowType, XmSHADOW_IN,
			  XmNwidth, BW,
			  XmNheight, BH,
			  NULL);
	} else {
	    XtVaSetValues(bt[i],
			  XmNshadowType, XmSHADOW_OUT,
			  XmNpushButtonEnabled, True,
			  XmNwidth, BW,
			  XmNheight, BH,
			  NULL);
	}
	XtManageChild(bt[i]);
    } else {
	XtUnmanageChild(bt[i]);
    }
}

static void stamp_cb(Widget w, XtPointer client_data, XmDrawnButtonCallbackStruct * s)
{
    int n;
    int old;

    XtVaGetValues(w,
		  XmNuserData, &n,
		  NULL);

    old = selected;
    selected = n + offset;

    if (old != -1 && old-offset < WB*HB && old-offset >= 0) {
	redraw_stamp(old-offset);
    }
    redraw_stamp(selected-offset);
}

static void create_button(int n)
{
    Position x, y;
    Widget w;

    x = n / 3;
    y = n % 3;

    x = x * BW;
    y = y * BH + 18;

    w = XtVaCreateWidget("pict", xmDrawnButtonWidgetClass, bul,
			 XmNlabelType, XmPIXMAP,
			 XmNhighlightThickness, 0,
			 XmNshadowThickness, 2,
			 XmNpushButtonEnabled, True,
			 XmNshadowType, XmSHADOW_ETCHED_IN,
			 XmNuserData, n,
			 XmNwidth, BW,
			 XmNheight, BH,
			 XmNx, x,
			 XmNy, y,
			 NULL);
    bt[n] = w;
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)stamp_cb, (void *) NULL);

#ifdef DND
    stamp_drag_translations = XtParseTranslationTable(dragTranslations);
    XtOverrideTranslations(w, stamp_drag_translations);
#endif
}


static redraw_stamps()
{
    int i;

    for (i = 0; i < (HB * WB); i++)
	redraw_stamp(i);
}


static void decrease_cb()
{
    offset -= 3;
    if (offset < 0)
	offset = 0;

    redraw_stamps();
}

static void increase_cb()
{
    if (!(offset + 3 > NUMOBJ - (HB * WB) + HB))
	offset += 3;

    redraw_stamps();
}

unmap_cb()
{
    XtPopdown(dialog);
    /* make sure the dialog goes away before returning. Sync with server
     * and update the display.
     */
    XSync(XtDisplay(dialog), 0);
    XmUpdateDisplay(XtParent(dialog));
    runing = 0;
    selected = -1;
}

void show_stamps()
{
    Arg args[100];
    int n;
    XmString s;

    if (!dialog) {
	s = XmStringCreateSimple("");
	dialog = XtVaCreatePopupShell("stamps",
			 xmDialogShellWidgetClass, GetTopShell(toplevel),
				      XmNdeleteResponse, XmUNMAP,
				      XmNresizePolicy, XmRESIZE_NONE,
				      XmNnoResize, True,
				      XmNwidth, W,
				      XmNminWidth, W,
				      XmNmaxWidth, W,
				      XmNheight, H,
				      XmNminHeight, H,
				      XmNmaxHeight, H,
				      XmNmarginWidth, 5,
				      XmNmarginHeight, 5,
				      NULL);

	bul = XtVaCreateWidget("bul",
			       xmBulletinBoardWidgetClass, dialog,
			       XmNmarginHeight, 0,
			       XmNmarginWidth, 0,
			       XmNshadowThickness, 0,
			       XmNautoUnmanage, False,
			       XmNdialogStyle, XmDIALOG_WORK_AREA,
				      XmNheight, H,
				      XmNwidth, W,
			       NULL);
	XtAddCallback(bul, XmNunmapCallback, (void *) unmap_cb, NULL);


	ab1 = XtVaCreateWidget("abl",
			       xmArrowButtonWidgetClass, bul,
			       XmNarrowDirection, XmARROW_LEFT,
			       XmNhighlightThickness, 0,
			       XmNwidth, 16,
			       XmNheight, 16,
			       XmNx, 0,
			       XmNy, 0,
			       NULL);
	XtAddCallback(ab1, XmNactivateCallback, (void *) decrease_cb, NULL);

	ab2 = XtVaCreateWidget("abr",
			       xmArrowButtonWidgetClass, bul,
			       XmNarrowDirection, XmARROW_RIGHT,
			       XmNhighlightThickness, 0,
			       XmNwidth, 16,
			       XmNheight, 16,
			       XmNx, 3 * BW + (BW - 16),
			       XmNy, 0,
			       NULL);
	XtAddCallback(ab2, XmNactivateCallback, (void *) increase_cb, NULL);


	for (n = 0; n < (HB * WB); n++)
	    create_button(n);

	XmStringFree(s);


    }
    if (!runing) {
	runing = 1;
	XtManageChild(ab1);
	XtManageChild(ab2);
	XtManageChild(bul);
	XtManageChild(dialog);
	XtPopup(dialog, XtGrabNone);

	redraw_stamps();
    } else {
	unmap_cb();
    }
}

#ifdef DND
void DragDropFinish(Widget w, XtPointer client_data, XtPointer call_data)
{
    Widget si = NULL;

    XtVaGetValues(w, XmNsourceCursorIcon, &si, NULL);
    if (si) {
	XtDestroyWidget(si);
	fprintf(stderr, "Destroyed Cursor!\n");
    }
    XtVaGetValues(w, XmNsourcePixmapIcon, &si, NULL);
    if (si) {
	XtDestroyWidget(si);
	fprintf(stderr, "Destroyed Pixmap!\n");
    }
    fprintf(stderr, "Drop Finished!\n");
}
#endif

#ifdef DND
StampStartDrag(Widget widget, XEvent * event, String * params, Cardinal * num_params)
{
    Arg args[10];
    int n, i;
    Display *dpy;
    Atom exportList[1];
    Widget drag_icon, dc;
    Pixel fg, bg;
    Pixmap icon, iconmask;
    XtPointer ptr;
    extern Atom PIXMAP_INDEX;

    /* intern the Atoms for data targets */
    dpy = XtDisplay(widget);

    /* get background and foreground colors and fetch index into file
     * array from XmNuserData.
     */
    XtVaGetValues(widget,
		  XmNbackground, &bg,
		  XmNforeground, &fg,
		  XmNuserData, &ptr,
		  NULL);

    /* create pixmaps for drag icon -- either file or directory */
    i = (int) ptr + offset;

    n = 0;
    XtSetArg(args[n], XmNpixmap, objects[i].pmap);
    n++;
    XtSetArg(args[n], XmNmask, objects[i].smap);
    n++;
    XtSetArg(args[n], XmNwidth, 16);
    n++;
    XtSetArg(args[n], XmNheight, 16);
    n++;
    XtSetArg(args[n], XmNdepth, DefaultDepthOfScreen(XtScreen(widget)));
    n++;
    drag_icon = XmCreateDragIcon(widget, "drag_icon", args, n);

    /* specify resources for DragContext for the transfer */
    n = 0;
    XtSetArg(args[n], XmNblendModel, XmBLEND_JUST_SOURCE);
    n++;
    XtSetArg(args[n], XmNsourcePixmapIcon, drag_icon);
    n++;

    /* establish the list of valid target types */
    exportList[0] = PIXMAP_INDEX;
    XtSetArg(args[n], XmNexportTargets, exportList);
    n++;
    XtSetArg(args[n], XmNnumExportTargets, 1);
    n++;

    XtSetArg(args[n], XmNdragOperations, XmDROP_COPY);
    n++;
    XtSetArg(args[n], XmNclientData, (XtPointer) "hallo");
    n++;

    /* start the drag and register a callback to clean up when done */
    dc = XmDragStart(widget, event, args, n);
    XtAddCallback(dc, XmNdragDropFinishCallback, DragDropFinish, NULL);
}

#endif
